﻿using ComputersShopContracts.BindingModels;
using ComputersShopContracts.BusinessLogicContracts;
using ComputersShopContracts.SearchModels;
using ComputersShopContracts.StoragesContracts;
using ComputersShopContracts.ViewModels;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ComputersShopBusinessLogic.BusinessLogics
{
    public class ComputerLogic : IComputerLogic
    {
        private readonly ILogger _logger;
        private readonly IComputerStorage _computerStorage;

        public ComputerLogic(ILogger<ComputerLogic> logger, IComputerStorage computerStorage)
        {
            _logger = logger;
            _computerStorage = computerStorage;
        }

        public List<ComputerViewModel>? ReadList(ComputerSearchModel? model)
        {
            _logger.LogInformation("ReadList. ComputerName:{ComputerName}.Id:{ Id}", model?.ComputerName, model?.Id);
            var list = model == null ? _computerStorage.GetFullList() : _computerStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        public ComputerViewModel? ReadElement(ComputerSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. ComputerName:{ComputerName}.Id:{ Id}", model.ComputerName, model.Id);
            var element = _computerStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }

        public bool Create(ComputerBindingModel model)
        {
            CheckModel(model);
            if (_computerStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }

        public bool Update(ComputerBindingModel model)
        {
            CheckModel(model);
            if (_computerStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }

        public bool Delete(ComputerBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id:{Id}", model.Id);
            if (_computerStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }

        private void CheckModel(ComputerBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (string.IsNullOrEmpty(model.ComputerName))
            {
                throw new ArgumentNullException("Нет названия компьютера", nameof(model.ComputerName));
            }
            if (model.Price <= 0)
            {
                throw new ArgumentNullException("Цена компьютера должна быть больше 0", nameof(model.Price));
            }
            _logger.LogInformation("Computer. ComputerName:{ComputerName}.Cost:{ Cost}. Id: { Id}", model.ComputerName, model.Price, model.Id);
            var element = _computerStorage.GetElement(new ComputerSearchModel
            {
                ComputerName = model.ComputerName
            });
            if (element != null && element.Id != model.Id)
            {
                throw new InvalidOperationException("Компьютер с таким названием уже есть");
            }
        }
    }
}
